#include <stdio.h>
#include <string.h>
#include "tk.h"
#include <errno.h>
#include <net/ppp_defs.h>
#include <net/if.h>
#include <net/if_ppp.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "Info.h"
#include "Common.h"

#define FALSE 0
#define TRUE 1

extern Tcl_Interp *myinterp;

int s = -1;			/* socket file descriptor */
struct ppp_stats old;
char fifoname[] = "/usr/tmp/startppp_pipe";
int pipeinfofd;
int infochildpid = -1;
int helperchildpid = -1;
/*
**	Function name : GetTransfer
**
**	Description :
**	Input :
**	Output :
*/
int GetTransfer(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{
  char buf[10];
  register int line = 0;
  struct ppp_stats cur;
  
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }
  
  if (s == -1) {
    return TCL_OK;
  }

  if (get_stats(&cur) == -1) {
    return TCL_OK;
  }

  sprintf(buf,"%8d",(line % 20? cur.p.ppp_ibytes - old.p.ppp_ibytes: cur.p.ppp_ibytes));
  Tcl_SetVar(myinterp, "getbytes", buf, TCL_GLOBAL_ONLY);
  if ((line % 20? cur.p.ppp_ibytes - old.p.ppp_ibytes: cur.p.ppp_ibytes) != 0) {
    Tcl_SetVar(myinterp, "getbytessave", buf, TCL_GLOBAL_ONLY);
  }
  sprintf(buf,"%8d",(line % 20? cur.p.ppp_obytes - old.p.ppp_obytes: cur.p.ppp_obytes));
  Tcl_SetVar(myinterp, "sendbytes", buf, TCL_GLOBAL_ONLY);
  if ((line % 20? cur.p.ppp_obytes - old.p.ppp_obytes: cur.p.ppp_obytes) != 0) {
    Tcl_SetVar(myinterp, "sendbytessave", buf, TCL_GLOBAL_ONLY);
  }
    
  return TCL_OK;

}

/*
**	Function name : TransferUp
**
**	Description :
**	Input :
**	Output :
*/
int TransferUp(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  if (s != -1) {
    return TCL_OK;
  }

  if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    perror("couldn't create IP socket");
    s == -1;
    return TCL_OK;
  }

  memset(&old, 0, sizeof(old));

  Tcl_SetVar(myinterp, "transferstatus", "up", TCL_GLOBAL_ONLY);
  return TCL_OK;
}

/*
**	Function name : TransferDown
**
**	Description :
**	Input :
**	Output :
*/
int TransferDown(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  if (s == -1) {
    return TCL_OK;
  }

  Tcl_SetVar(myinterp, "transferstatus", "down", TCL_GLOBAL_ONLY);
  if (shutdown(s, 2) == -1) {
    if (errno == EBADF) {
    }
    if (errno == ENOTSOCK) {
    }
    if (errno == ENOTCONN) {
    }

  }
  s = -1;

  return TCL_OK;
}

/*
**	Function name : get_stats
**
**	Description :
**	Input :
**	Output :
*/
int get_stats(struct ppp_stats *curp) {
  struct ifpppstatsreq req;
  
  memset (&req, 0, sizeof (req));
  
  req.stats_ptr = (caddr_t) &req.stats;
  
  sprintf(req.ifr__name, "ppp0");
  if (ioctl(s, SIOCGPPPSTATS, &req) < 0) {
    if (errno == ENOTTY)
      fprintf(stderr, "ppp0: kernel support missing\n");
    return -1;
  }
  *curp = req.stats;
  
  return 0;
}

/*
**	Function name : CreateInfoPipe
**
**	Description :
**	Input :
**	Output :
*/
int CreateInfoPipe(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{

#define FIFO_MODE (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)
  FILE *pipefd2;
  
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  unlink(fifoname);

  if (mkfifo(fifoname, FIFO_MODE) < 0) {
    fprintf(stderr, "mkfifo: not possible\n");
    /*    return TCL_OK; */
  }

  /* Create Filehandler */
  
  if ((pipeinfofd=open(fifoname,O_RDWR|O_NONBLOCK)) < 1) {
    fprintf(stderr, "open of fifo: not possible\n");
    return TCL_OK;
  }
  Tk_CreateFileHandler(pipeinfofd,TK_READABLE,PipeReadProc,(ClientData) NULL); 

  return TCL_OK;
}

/*
**	Function name : DeleteInfoPipe
**
**	Description :
**	Input :
**	Output :
*/
int DeleteInfoPipe(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  unlink(fifoname);

  return TCL_OK;
}

/*
   **	Function name : PipeReadProc
   **
   **	Description :
   **	Input :
   **	Output :
*/
void PipeReadProc(ClientData clientData, int mask)
{
  int count;
  char input[200];
  char t1[20];
  char t2[20];
  char t3[20];
  char t4[20];
  char t5[20];
  char t6[20];
  char i_up[] = "interface_up";
  char i_down[]  = "interface_down";
  char i_show[] = "showppp";
  
  if ((count=read(pipeinfofd, input, 200))<1) {
    return;
  }

  input[count]='\0';

  if (strstr(input, i_up)!=NULL) {
    sscanf(input,"%s %s %s %s %s %s",t1,t2,t3,t4,t5,t6);
    Tcl_SetVar(myinterp, "infostate", t1, TCL_GLOBAL_ONLY);
    Tcl_SetVar(myinterp, "ifname",    t2, TCL_GLOBAL_ONLY);
    Tcl_SetVar(myinterp, "ttydev",    t3, TCL_GLOBAL_ONLY);
    Tcl_SetVar(myinterp, "pppspeed",  t4, TCL_GLOBAL_ONLY);
    Tcl_SetVar(myinterp, "localip",   t5, TCL_GLOBAL_ONLY);
    Tcl_SetVar(myinterp, "remoteip",  t6, TCL_GLOBAL_ONLY);
    Tcl_Eval (myinterp, i_show);
  }

  if (strstr(input, i_down)!=NULL) {
    sscanf(input,"%s %s %s %s %s %s",t1,t2,t3,t4,t5,t6);
    Tcl_SetVar(myinterp, "infostate", t1, TCL_GLOBAL_ONLY);
    Tcl_SetVar(myinterp, "ifname",    t2, TCL_GLOBAL_ONLY);
    Tcl_SetVar(myinterp, "ttydev",    t3, TCL_GLOBAL_ONLY);
    Tcl_SetVar(myinterp, "pppspeed",  t4, TCL_GLOBAL_ONLY);
    Tcl_SetVar(myinterp, "localip",   t5, TCL_GLOBAL_ONLY);
    Tcl_SetVar(myinterp, "remoteip",  t6, TCL_GLOBAL_ONLY);
    Tcl_Eval (myinterp, i_show);
  }

  return;
}

/*
**	Function name : InfoUp
**
**	Description :
**	Input :
**	Output :
*/
int InfoUp(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{
  int pid;

  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  /* forking */
  if ((pid = fork()) > 0) {  /* parent process  */
    /* set infochildpid on pid */
    infochildpid=pid;
    return TCL_OK;
  }
  else {
    if (pid == 0) {  /* child process */
      char *infoprog;
      char *args[10];
      char *work;
      int i = 0;
      int x = 0;
      int y = 0;
      int in = FALSE;
      
      infoprog=Tcl_GetVar(myinterp, "infoprog", TCL_GLOBAL_ONLY);

      while (i < strlen(infoprog)) {
	if (isspace(infoprog[i]) != 0) { /* ist ein Leerzeichen */
	  if (in == TRUE) {
	    in = FALSE;
	    work[y] = '\0';
	    args[x] = work;
	    x++;
	    y = 0;
	  }
	} else {   /* kein Leerzeichen */
	  if (in == FALSE) {  /* war vorher aussen */
	    in = TRUE;
	    work = (char *)malloc(100);
	  }	  
	  work[y]=infoprog[i];
	  y++;
	}
      	i++;
      }
      if (i != 0) {
	work[y] = '\0';
	args[x] = work;
      }
      args[x+1] = NULL;
      
      /*
      printf("%s  %d    %d\n",infoprog,x,y);
      i=0;
      while (i <= x) {
	printf("%s\n",args[i]);
	i++;
      } 
      */
      execvp(args[0], args);
      printf("something went wrong\n");
    }
    else { /* <fork> error */
      printf("fork error\n");
      return;
    }
  }
  
  return TCL_OK;
  
  
}

/*
**	Function name : InfoDown
**
**	Description :
**	Input :
**	Output :
*/
int InfoDown(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }
  
  if (infochildpid == -1) {
    return TCL_OK;
  }

  if (kill(infochildpid,9)==0) {
    infochildpid = -1;
    return TCL_OK;
  }
  
  return TCL_OK;


}

/*
**	Function name : HelperUp
**
**	Description :
**	Input :
**	Output :
*/
int HelperUp(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{
  int pid;

  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  /* forking */
  if ((pid = fork()) > 0) {  /* parent process  */
    /* set helperchildpid on pid */
    helperchildpid=pid;
    return TCL_OK;
  }
  else {
    if (pid == 0) {  /* child process */
      char *ppp_up;
      char *args[20];
      char *work;
      int i = 0;
      int x = 0;
      int y = 0;
      int in = FALSE;
      
      ppp_up=Tcl_GetVar(myinterp, "ppp_up", TCL_GLOBAL_ONLY);

      while (i < strlen(ppp_up)) {
	if (isspace(ppp_up[i]) != 0) { /* ist ein Leerzeichen */
	  if (in == TRUE) {
	    in = FALSE;
	    work[y] = '\0';
	    args[x] = work;
	    x++;
	    y = 0;
	  }
	} else {   /* kein Leerzeichen */
	  if (in == FALSE) {  /* war vorher aussen */
	    in = TRUE;
	    work = (char *)malloc(100);
	  }	  
	  work[y]=ppp_up[i];
	  y++;
	}
      	i++;
      }
      if (i != 0) {
	work[y] = '\0';
	args[x] = work;
      }
      args[x+1] = NULL;
      
      execvp(args[0], args);
      printf("ppp_up shouldn't come back!\n");
    }
    else { /* <fork> error */
      printf("fork error\n");
      return;
    }
  }
  
  return TCL_OK;
  
  
}

/*
**	Function name : HelperDown
**
**	Description :
**	Input :
**	Output :
*/
int HelperDown(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }
  
  if (helperchildpid == -1) {
    return TCL_OK;
  }

  if (kill(helperchildpid,15)==0) {
    helperchildpid = -1;
    return TCL_OK;
  }
  
  return TCL_OK;


}
